@@ -1,7 +1,7 @@
 /*
  * chcpumf -  Change CPU Measurement Facility Characteristics
  *
- * Copyright IBM Corp. 2020
+ * Copyright IBM Corp. 2020, 2022
  *
  * s390-tools is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -20,9 +20,9 @@
 #include "lib/util_prg.h"
 #include "lib/util_base.h"
 
-#include "defines.h"
+#include "lib/libcpumf.h"
 
-static int verbose;
+static unsigned int verbose;
 static unsigned long min_sdb, max_sdb;
 
 static struct util_opt opt_vec[] = {
@@ -85,62 +85,31 @@ static long parse_buffersize(char *string)
 	return bytes;
 }
 
-static int read_sfb(unsigned long *min, unsigned long *max)
-{
-	unsigned long cur_min_sdb, cur_max_sdb;
-	int rc = EXIT_SUCCESS;
-	FILE *fp;
-
-	if (geteuid()) {
-		fprintf(stderr, "Error: Must run as root\n");
-		return EXIT_FAILURE;
-	}
-	fp = fopen(PERF_SFB_SIZE, "r");
-	if (fp == NULL) {
-		linux_error(PERF_SFB_SIZE);
-		return EXIT_FAILURE;
-	}
-	if (fscanf(fp, "%ld,%ld", &cur_min_sdb, &cur_max_sdb) != 2) {
-		fprintf(stderr, "Error: Can not parse file " PERF_SFB_SIZE
-				"\n");
-		rc = EXIT_FAILURE;
-	} else {
-		if (*min == 0)
-			*min = cur_min_sdb;
-		if (*max == 0)
-			*max = cur_max_sdb;
-	}
-	fclose(fp);
-	return rc;
-}
-
-static int write_sfb(unsigned long min, unsigned long max)
+static int write_sfb(unsigned int min, unsigned int max)
 {
 	int rc = EXIT_SUCCESS;
 	char text[64];
 	size_t len;
 	FILE *fp;
 
-	fp = fopen(PERF_SFB_SIZE, "w");
-	if (fp == NULL) {
-		linux_error(PERF_SFB_SIZE);
-		return EXIT_FAILURE;
-	}
-	snprintf(text, sizeof text, "%ld,%ld", min, max);
+	fp = fopen(S390_CPUMSF_BUFFERSZ, "w");
+	if (!fp)
+		err(EXIT_FAILURE, S390_CPUMSF_BUFFERSZ);
+	snprintf(text, sizeof(text), "%u,%u", min, max);
 	len = strlen(text) + 1;
 	if (fwrite(text, 1, len, fp) != len) {
-		linux_error(PERF_SFB_SIZE);
+		warn(S390_CPUMSF_BUFFERSZ);
 		rc = EXIT_FAILURE;
 	}
 	if (fclose(fp)) {
-		linux_error(PERF_SFB_SIZE);
+		warn(S390_CPUMSF_BUFFERSZ);
 		rc = EXIT_FAILURE;
 	}
 	if (verbose && rc != EXIT_FAILURE)
-		fprintf(stderr, "Sampling buffer sizes:\n"
-				"    Minimum:%7ld sample-data-blocks\n"
-				"    Maximum:%7ld sample-data-blocks\n",
-				min, max);
+		warnx("Sampling buffer sizes:\n"
+		      "    Minimum:%7d sample-data-blocks\n"
+		      "    Maximum:%7d sample-data-blocks\n",
+		      min, max);
 	return rc;
 }
 
@@ -160,61 +129,50 @@ static int parse_args(int argc, char **argv)
 			exit(EXIT_SUCCESS);
 		case 'x':
 			new = parse_buffersize(optarg);
-			if (new < 1) {
-				fprintf(stderr, "The specified number(s)"
-						" are not valid\n");
-				exit(EXIT_FAILURE);
-			}
+			if (new < 1)
+				errx(EXIT_FAILURE,
+				     "The specified number(s) are not valid");
 			max_sdb = new;
 			action = 1;
 			break;
 		case 'm':
 			new = parse_buffersize(optarg);
-			if (new < 1) {
-				fprintf(stderr, "The specified number(s)"
-						" are not valid\n");
-				exit(EXIT_FAILURE);
-			}
+			if (new < 1)
+				errx(EXIT_FAILURE,
+				     "The specified number(s) are not valid");
 			min_sdb = new;
 			action = 1;
 			break;
 		case 'V':
 			verbose = 1;
 			break;
-		case '?':
-			fprintf(stderr, "One or more options are not valid\n");
-			fprintf(stderr, "Try 'chcpumf --help' for more"
-					" information\n");
+		default:
+			util_opt_print_parse_error(opt, argv);
 			exit(EXIT_FAILURE);
 		}
 	}
-	if (!action) {
-		fprintf(stderr, "You must specify a valid option\n");
-		exit(EXIT_FAILURE);
-	}
+	if (!action)
+		errx(EXIT_FAILURE, "You must specify a valid option");
 	return action;
 }
 
 int main(int argc, char **argv)
 {
-	int ret = EXIT_FAILURE;
-	struct stat sbuf;
+	unsigned long my_min, my_max;
 
 	util_prg_init(&prg);
 	util_opt_init(opt_vec, NULL);
 
 	parse_args(argc, argv);
-	if (stat(PERF_PATH PERF_SF, &sbuf) != 0) {
-		fprintf(stderr,
-			"No CPU-measurement sampling facility detected\n");
-		return ret;
-	}
-	if (read_sfb(&min_sdb, &max_sdb))
-		return ret;
-	if (min_sdb >= max_sdb) {
-		fprintf(stderr, "The specified maximum must be greater "
-				"than the minimum\n");
-		return ret;
-	}
+	if (geteuid())
+		errx(EXIT_FAILURE, "Must run as root");
+	if (!libcpumf_have_sfb())
+		errx(EXIT_FAILURE,
+		     "No CPU-measurement sampling facility detected");
+	libcpumf_sfb_info(&my_min, &my_max);
+	if (!min_sdb)
+		min_sdb = my_min;
+	if (!max_sdb)
+		max_sdb = my_max;
 	return write_sfb(min_sdb, max_sdb);
 }
