@@ -33,6 +33,9 @@
 #include "lib/util_path.h"
 #include "lib/util_udev.h"
 
+static const char default_mask[AP_MASK_SIZE] =
+	"0x0000000000000000000000000000000000000000000000000000000000000000";
+
 /*
  * Return sysfs path to a bus attribute
  * Note: caller is responsible for freeing the returned string
@@ -118,6 +121,37 @@ static void modify_device_attr(struct util_list *list, char *value)
 	vfio_ap_node_remove_dupes(list);
 }
 
+/*
+ * Pass a comma-delimited string of masks (adapters,domains,controls) and
+ * for each ON bit in these masks add the associated ID to the device
+ * lists.
+ */
+static void modify_device_ap_config(struct vfio_ap_device *dev,
+				    char *value)
+{
+	char *mask, *adapters, *domains, *controls;
+
+	mask = util_strdup(value);
+
+	adapters = strtok(mask, ",");
+	domains = strtok(NULL, ",");
+	controls = strtok(NULL, ",");
+	util_assert((!strtok(NULL, ",")) && adapters && domains && controls,
+		    "Invalid ap_config attribute encountered %s", value);
+
+	/*
+	 * ap_config overwrites the current list of adapters, domains and
+	 * control domains.  Clear the current lists before generating new ones
+	 * based upon the input mask values.
+	 */
+	ap_list_remove_all(dev->adapters);
+	ap_list_remove_all(dev->domains);
+	ap_list_remove_all(dev->controls);
+	ap_mask_to_list(adapters, dev->adapters);
+	ap_mask_to_list(domains, dev->domains);
+	ap_mask_to_list(controls, dev->controls);
+}
+
 static void load_attr_to_device(struct vfio_ap_device *dev, char *attr,
 				const char *value)
 {
@@ -129,6 +163,8 @@ static void load_attr_to_device(struct vfio_ap_device *dev, char *attr,
 		modify_device_attr(dev->domains, v);
 	else if (strcmp(attr, "assign_control_domain") == 0)
 		modify_device_attr(dev->controls, v);
+	else if (strcmp(attr, "ap_config") == 0)
+		modify_device_ap_config(dev, v);
 
 	free(v);
 }
@@ -416,6 +452,28 @@ void vfio_ap_parse_control(struct vfio_ap_device *dev, char *control)
 	}
 }
 
+/**
+ * Determine if the specified device is currently active.  If so, see if
+ * it is enabled for dynamic configuration support.
+ *
+ * @param[in]      dev        Vfio-ap struct
+ *
+ * @retval         True       Device is active and enabled for dynamic config
+ * @retval         False      Device is not active OR no dynamic config support
+ */
+bool vfio_ap_need_dynamic_config(struct vfio_ap_device *dev)
+{
+	char *attr = path_get_vfio_ap_attr(dev->uuid, "ap_config");
+
+	if (!attr)
+		return false;
+
+	if (!util_path_is_readable(attr))
+		return false;
+
+	return true;
+}
+
 #ifdef HAVE_JSONC
 
 /**
@@ -692,7 +750,7 @@ void ap_mask_to_list(char *mask, struct util_list *list)
  */
 void ap_list_remove_all(struct util_list *list)
 {
-	struct ap_node *node;
+	struct vfio_ap_node *node;
 
 	while (!util_list_is_empty(list)) {
 		node = util_list_start(list);
@@ -715,6 +773,90 @@ static unsigned int random_delay(void)
 	return AP_LOCK_DELAY_US + (rand() % AP_LOCK_VARIANCE_US);
 }
 
+/**
+ * Return a mask of assigned adapters for the specified vfio_ap device.
+ * Note: caller is responsible for freeing the returned string
+ *
+ * @param[in]      dev        Vfio-ap struct to get adapter mask from
+ * @param[in, out] size       Size of mask buffer returned
+ *
+ * @retval         != 0       Adapter mask (hex string)
+ * @retval         0          Failed to generate a mask
+ */
+char *vfio_ap_device_get_adapter_mask(struct vfio_ap_device *dev, int *size)
+{
+	struct vfio_ap_node *node;
+	char *mask;
+
+	if (!dev || !size)
+		return NULL;
+
+	mask = util_strdup(default_mask);
+	*size = AP_MASK_SIZE;
+
+	util_list_iterate(dev->adapters, node) {
+		ap_set_bit(node->id, mask, true);
+	}
+
+	return mask;
+}
+
+/**
+ * Return a mask of assigned domains for the specified vfio_ap device.
+ * Note: caller is responsible for freeing the returned string
+ *
+ * @param[in]      dev        Vfio-ap struct to get domain mask from
+ * @param[in, out] size       Size of mask buffer returned
+ *
+ * @retval         != 0       Domain mask (hex string)
+ * @retval         0          Failed to generate a mask
+ */
+char *vfio_ap_device_get_domain_mask(struct vfio_ap_device *dev, int *size)
+{
+	struct vfio_ap_node *node;
+	char *mask;
+
+	if (!dev || !size)
+		return NULL;
+
+	mask = util_strdup(default_mask);
+	*size = AP_MASK_SIZE;
+
+	util_list_iterate(dev->domains, node) {
+		ap_set_bit(node->id, mask, true);
+	}
+
+	return mask;
+}
+
+/**
+ * Return a mask of assigned control domains for the specified vfio_ap device.
+ * Note: caller is responsible for freeing the returned string
+ *
+ * @param[in]      dev        Vfio-ap struct to get control domain mask from
+ * @param[in, out] size       Size of mask buffer returned
+ *
+ * @retval         != 0       Control domain mask (hex string)
+ * @retval         0          Failed to generate a mask
+ */
+char *vfio_ap_device_get_control_mask(struct vfio_ap_device *dev, int *size)
+{
+	struct vfio_ap_node *node;
+	char *mask;
+
+	if (!dev || !size)
+		return NULL;
+
+	mask = util_strdup(default_mask);
+	*size = AP_MASK_SIZE;
+
+	util_list_iterate(dev->controls, node) {
+		ap_set_bit(node->id, mask, true);
+	}
+
+	return mask;
+}
+
 /**
  * Acquire the ap config lock using this Process ID
  *
