@@ -7,10 +7,11 @@ use std::{
     mem::size_of,
 };
 
+use log::{debug, warn};
+use zerocopy::{AsBytes, BigEndian, FromBytes, FromZeroes, U32, U64};
+
 // (SE) boot request control block aka SE header
 use crate::{assert_size, request::MagicValue, static_assert, Error, Result, PAGESIZE};
-use log::debug;
-use zerocopy::{AsBytes, BigEndian, FromBytes, FromZeroes, U32, U64};
 
 /// Struct containing all SE-header tags.
 ///
@@ -45,6 +46,81 @@ impl TryFrom<Vec<u8>> for BootHdrTags {
     }
 }
 
+/// Struct representing the Secure Execution boot image metadata
+#[allow(unused)]
+#[repr(packed)]
+#[derive(Debug, Clone, FromBytes, FromZeroes, AsBytes, PartialEq, Eq)]
+pub struct SeImgMetaData {
+    /// Magic value
+    magic: [u8; 8],
+    /// Secure Execution header offset in the image
+    hdr_off: U64<BigEndian>,
+    /// Version
+    version: U32<BigEndian>,
+    /// IPIB offset in the image
+    ipib_off: U64<BigEndian>,
+}
+assert_size!(SeImgMetaData, 28);
+
+impl SeImgMetaData {
+    /// Address in the Secure Execution boot image
+    pub const OFFSET: u64 = 0xc000;
+    /// V1 of the Secure Execution boot image metadata
+    const V1: u32 = 0x1;
+
+    /// Create v1 Secure Execution image metadata.
+    pub fn new_v1(hdr_off: u64, ipib_off: u64) -> Self {
+        Self {
+            magic: Self::MAGIC,
+            version: Self::V1.into(),
+            hdr_off: hdr_off.into(),
+            ipib_off: ipib_off.into(),
+        }
+    }
+
+    fn seek_start<R>(img: &mut R) -> Result<bool>
+    where
+        R: Read + Seek,
+    {
+        const BUF_SIZE: i64 = 8;
+        static_assert!(SeImgMetaData::MAGIC.len() == BUF_SIZE as usize);
+
+        let mut buf = [0; BUF_SIZE as usize];
+        match img.seek(std::io::SeekFrom::Start(Self::OFFSET)) {
+            Ok(it) => it,
+            Err(_) => return Ok(false),
+        };
+        match img.read_exact(&mut buf) {
+            Ok(it) => it,
+            Err(_) => return Ok(false),
+        }
+
+        if Self::starts_with_magic(&buf) {
+            // go back to the beginning of the metadata
+            img.seek(Current(-BUF_SIZE))?;
+            return Ok(true);
+        }
+        Ok(false)
+    }
+
+    /// Gets the bytes of this value.
+    #[inline(always)]
+    pub fn as_bytes(&self) -> &[u8] {
+        <Self as AsBytes>::as_bytes(self)
+    }
+
+    /// Returns the version of this [`SeImgMetaData`].
+    pub fn version(&self) -> u32 {
+        self.version.into()
+    }
+}
+
+/// Magic value for the metadata of a Secure Execution boot image
+impl MagicValue<8> for SeImgMetaData {
+    // ASCII `SeImgLnx`
+    const MAGIC: [u8; 8] = [0x53, 0x65, 0x49, 0x6d, 0x67, 0x4c, 0x6e, 0x78];
+}
+
 /// Magic value for a SE-(boot)header
 #[derive(Debug)]
 pub struct BootHdrMagic;
@@ -52,6 +128,65 @@ impl MagicValue<8> for BootHdrMagic {
     const MAGIC: [u8; 8] = [0x49, 0x42, 0x4d, 0x53, 0x65, 0x63, 0x45, 0x78];
 }
 
+/// Tries to seek to the start of the Secure Execution header.
+///
+/// Returns `false` if no Secure Execution header found, `true` otherwise.
+///
+/// # Errors
+///
+/// In the very unlikely case an IO error can appear when seeking to the
+/// beginning of the header.
+pub fn seek_se_hdr_start<R>(img: &mut R) -> Result<bool>
+where
+    R: Read + Seek,
+{
+    let max_iter: usize;
+    const BUF_SIZE: i64 = 8;
+    static_assert!(BootHdrMagic::MAGIC.len() == BUF_SIZE as usize);
+
+    let old_position = img.stream_position()?;
+    if !SeImgMetaData::seek_start(img)? {
+        // Search from the previous position.
+        img.seek(std::io::SeekFrom::Start(old_position))?;
+        max_iter = 0x15;
+    } else {
+        let mut img_metadata_bytes = vec![0u8; size_of::<SeImgMetaData>()];
+        // read in the header
+        img.read_exact(&mut img_metadata_bytes)?;
+        // Cannot fail because the buffer has the same size as SeImgMetaData.
+        let img_metadata = SeImgMetaData::ref_from(&img_metadata_bytes).unwrap();
+        let img_metadata_version = img_metadata.version();
+        if img_metadata_version != SeImgMetaData::V1 {
+            warn!("Unknown Secure Execution boot image version {img_metadata_version}");
+        }
+
+        img.seek(std::io::SeekFrom::Start(img_metadata.hdr_off.into()))?;
+        max_iter = 1;
+    }
+
+    let mut buf = [0; BUF_SIZE as usize];
+    for _ in 0..max_iter {
+        match img.read_exact(&mut buf) {
+            Ok(it) => it,
+            Err(_) => return Ok(false),
+        };
+
+        if BootHdrMagic::starts_with_magic(&buf) {
+            // go back to the beginning of the header
+            img.seek(Current(-BUF_SIZE))?;
+
+            return Ok(true);
+        }
+        // goto next page start
+        // or report invalid file format if file ends "early"
+        match img.seek(Current(PAGESIZE as i64 - BUF_SIZE)) {
+            Ok(it) => it,
+            Err(_) => return Ok(false),
+        };
+    }
+    Ok(false)
+}
+
 impl BootHdrTags {
     /// Returns a reference to the SE-header tag of this [`BootHdrTags`].
     pub fn tag(&self) -> &[u8; 16] {
@@ -64,40 +199,6 @@ impl BootHdrTags {
         Self { ald, tld, pld, tag }
     }
 
-    /// Returns `false` if no SE-header found, `true` otherwise.
-    /// In the very unlikely case an IO error can appear
-    /// when seeking to the beginning of the header.
-    fn seek_se_hdr_start<R>(img: &mut R) -> Result<bool>
-    where
-        R: Read + Seek,
-    {
-        const MAX_ITER: usize = 0x15;
-        const BUF_SIZE: i64 = 8;
-        static_assert!(BootHdrMagic::MAGIC.len() == BUF_SIZE as usize);
-
-        let mut buf = [0; BUF_SIZE as usize];
-        for _ in [0; MAX_ITER] {
-            match img.read_exact(&mut buf) {
-                Ok(it) => it,
-                Err(_) => return Ok(false),
-            };
-
-            if BootHdrMagic::starts_with_magic(&buf) {
-                // go back to the beginning of the header
-                img.seek(Current(-BUF_SIZE))?;
-
-                return Ok(true);
-            }
-            // goto next page start
-            // or report invalid file format if file ends "early"
-            match img.seek(Current(PAGESIZE as i64 - BUF_SIZE)) {
-                Ok(it) => it,
-                Err(_) => return Ok(false),
-            };
-        }
-        Ok(false)
-    }
-
     /// Deserializes a (SE) boot header and extracts the tags.
     ///
     /// Searches for the header; if found extracts the tags.
@@ -110,7 +211,7 @@ impl BootHdrTags {
     where
         R: Read + Seek,
     {
-        if !Self::seek_se_hdr_start(img)? {
+        if !seek_se_hdr_start(img)? {
             debug!("No boot hdr found");
             return Err(Error::InvBootHdr);
         }
@@ -149,7 +250,7 @@ impl BootHdrTags {
         let mut tag = [0u8; BootHdrHead::TAG_SIZE];
         img.read_exact(tag.as_mut_slice())?;
 
-        Ok(BootHdrTags {
+        Ok(Self {
             pld: hdr_head.pld,
             ald: hdr_head.ald,
             tld: hdr_head.tld,
@@ -186,8 +287,7 @@ mod tests {
     use std::io::Cursor;
 
     use super::*;
-    use crate::get_test_asset;
-    use crate::Error;
+    use crate::{get_test_asset, Error};
 
     const EXP_HDR: BootHdrTags = BootHdrTags {
         pld: [
@@ -279,4 +379,17 @@ mod tests {
         let der: Result<BootHdrTags> = ser.clone().try_into();
         assert!(matches!(der, Err(Error::InvBootHdrSize(_))));
     }
+
+    #[test]
+    fn se_img_metadata() {
+        let metadata = SeImgMetaData::new_v1(0x14000, 0x16000);
+        let data = [
+            83, 101, 73, 109, 103, 76, 110, 120, 0, 0, 0, 0, 0, 1, 64, 0, 0, 0, 0, 1, 0, 0, 0, 0,
+            0, 1, 96, 0,
+        ];
+        assert_eq!(metadata.as_bytes(), &data);
+        assert_eq!(SeImgMetaData::ref_from(&data), Some(&metadata));
+
+        assert_eq!(metadata.version(), SeImgMetaData::V1);
+    }
 }
