@@ -1,25 +1,29 @@
 // SPDX-License-Identifier: MIT
 //
 // Copyright IBM Corp. 2023
-
-use crate::assert_size;
-use crate::crypto::{
-    decrypt_aes_gcm, derive_key, encrypt_aes_gcm, gen_ec_key, random_array, AesGcmResult, SymKey,
-    SymKeyType, AES_256_GCM_TAG_SIZE,
-};
-use crate::misc::to_u32;
-use crate::request::Confidential;
-use crate::{Error, Result};
-use openssl::bn::{BigNum, BigNumContext};
-use openssl::ec::{EcGroupRef, EcPointRef};
-use openssl::error::ErrorStack;
-use openssl::hash::{hash, MessageDigest};
-use openssl::pkey::{PKey, PKeyRef, Private, Public};
-use pv_core::request::{RequestMagic, RequestVersion};
-use std::convert::TryInto;
 use std::mem::size_of;
+
+use openssl::{
+    bn::{BigNum, BigNumContext},
+    ec::{EcGroup, EcGroupRef, EcKey, EcPointRef},
+    error::ErrorStack,
+    hash::{DigestBytes, MessageDigest},
+    nid::Nid,
+    pkey::{PKey, PKeyRef, Private, Public},
+};
+use pv_core::request::{RequestMagic, RequestVersion};
 use zerocopy::{AsBytes, BigEndian, FromBytes, FromZeroes, U32};
 
+use crate::{
+    assert_size,
+    crypto::{
+        decrypt_aead, derive_aes256_gcm_key, encrypt_aead, gen_ec_key, hash, random_array,
+        AeadEncryptionResult, SymKey, SymKeyType,
+    },
+    misc::to_u32,
+    request::Confidential,
+    Error, Result,
+};
 /// Encrypt a _secret_ using self and a given private key.
 pub trait Encrypt {
     /// Encrypts `secret` using `self` and `priv_key` the encryption.
@@ -105,10 +109,10 @@ impl Encrypt for Keyslot {
         priv_key: &PKeyRef<Private>,
         to: &mut Vec<u8>,
     ) -> Result<()> {
-        let derived_key = derive_key(priv_key, &self.0)?;
+        let derived_key = derive_aes256_gcm_key(priv_key, &self.0)?;
         let mut wrpk_and_kst =
-            encrypt_aes_gcm(&derived_key.into(), &[0; 12], &[], prot_key)?.data();
-        let phk: EcdhPubkeyCoord = self.0.as_ref().try_into()?;
+            encrypt_aead(&derived_key.into(), &[0; 12], &[], prot_key)?.into_buf();
+        let phk: EcPubKeyCoord = self.0.as_ref().try_into()?;
 
         to.reserve(80);
         to.extend_from_slice(&hash(MessageDigest::sha256(), phk.as_ref())?);
@@ -144,11 +148,11 @@ impl ReqEncrCtx {
         S: Into<Option<SymKey>>,
     {
         let iv = iv.into().unwrap_or(random_array()?);
-        let priv_key = priv_key.into().unwrap_or(gen_ec_key()?);
+        let priv_key = priv_key.into().unwrap_or(gen_ec_key(Nid::SECP521R1)?);
         let prot_key = prot_key
             .into()
-            .unwrap_or(SymKey::random(SymKeyType::Aes256)?);
-        Ok(ReqEncrCtx {
+            .unwrap_or(SymKey::random(SymKeyType::Aes256Gcm)?);
+        Ok(Self {
             iv,
             priv_key,
             prot_key,
@@ -162,7 +166,8 @@ impl ReqEncrCtx {
     /// This function will return an error if OpenSSL could not generate a random value.
     pub fn random(ket_tp: SymKeyType) -> Result<Self> {
         match ket_tp {
-            SymKeyType::Aes256 => Self::new_aes_256(None, None, None),
+            SymKeyType::Aes256Gcm => Self::new_aes_256(None, None, None),
+            SymKeyType::Aes256Xts => Err(Error::NoAeadKey),
         }
     }
 
@@ -238,7 +243,7 @@ impl ReqEncrCtx {
     ///
     /// This function will return an error if the public key could not be extracted by OpenSSL.
     /// Very unlikely.
-    pub fn key_coords(&self) -> Result<EcdhPubkeyCoord> {
+    pub fn key_coords(&self) -> Result<EcPubKeyCoord> {
         self.priv_key.as_ref().try_into().map_err(Error::Crypto)
     }
 
@@ -256,8 +261,8 @@ impl ReqEncrCtx {
     /// # Errors
     ///
     /// This function will return an error if the data could not be encrypted by OpenSSL.
-    pub(crate) fn encrypt_aead(&self, aad: &[u8], conf: &[u8]) -> Result<AesGcmResult> {
-        encrypt_aes_gcm(&self.prot_key, &self.iv, aad, conf)
+    pub(crate) fn encrypt_aead(&self, aad: &[u8], conf: &[u8]) -> Result<AeadEncryptionResult> {
+        encrypt_aead(&self.prot_key, &self.iv, aad, conf)
     }
 
     /// Returns a reference to the request protection key of this [`ReqEncrCtx`].
@@ -266,31 +271,64 @@ impl ReqEncrCtx {
     }
 }
 
+/// Public key components of an [`openssl::ec::EcKey`] key.
 #[repr(C)]
 #[derive(Debug, Clone)]
-pub struct EcdhPubkeyCoord([u8; 160]);
-impl AsRef<[u8]> for EcdhPubkeyCoord {
+pub struct EcPubKeyCoord([u8; 160]);
+impl AsRef<[u8]> for EcPubKeyCoord {
     fn as_ref(&self) -> &[u8] {
         self.0.as_slice()
     }
 }
 
+const ECDH_PUB_KEY_COORD_POINT_SIZE: usize = 0x50;
+
+impl EcPubKeyCoord {
+    /// Returns the SHA256 hash of the [`EcPubKeyCoord`].
+    ///
+    /// If [`EcPubKeyCoord`] was built from a host-key, this value is the public host-key hash.
+    pub fn sha256(&self) -> Result<DigestBytes> {
+        hash(MessageDigest::sha256(), self.as_ref())
+    }
+
+    /// Construct a [`EcPubKeyCoord`]
+    ///
+    /// # Safety
+    /// This function is marked unsafe, because data not representing two EC points violates the
+    /// invariant of this struct.
+    pub unsafe fn from_data(data: [u8; 160]) -> Self {
+        EcPubKeyCoord(data)
+    }
+}
+
 /// Get the pub ECDH coordinates in the format the Ultravisor expects it:
 /// The two coordinates are padded to 80 bytes each.
 fn get_pub_ecdh_points(pkey: &EcPointRef, grp: &EcGroupRef) -> Result<[u8; 160], ErrorStack> {
-    const ECDH_PUB_KEY_COORD_POINT_SIZE: i32 = 0x50;
     let mut x = BigNum::new()?;
     let mut y = BigNum::new()?;
     let mut bn_ctx = BigNumContext::new()?;
     pkey.affine_coordinates(grp, &mut x, &mut y, &mut bn_ctx)?;
-    let mut coord: Vec<u8> = x.to_vec_padded(ECDH_PUB_KEY_COORD_POINT_SIZE)?;
-    coord.append(&mut y.to_vec_padded(ECDH_PUB_KEY_COORD_POINT_SIZE)?);
+    let mut coord: Vec<u8> = x.to_vec_padded(ECDH_PUB_KEY_COORD_POINT_SIZE as i32)?;
+    coord.append(&mut y.to_vec_padded(ECDH_PUB_KEY_COORD_POINT_SIZE as i32)?);
     Ok(coord.try_into().unwrap())
 }
 
+impl TryFrom<EcPubKeyCoord> for PKey<Public> {
+    type Error = ErrorStack;
+
+    fn try_from(value: EcPubKeyCoord) -> Result<Self, Self::Error> {
+        let ecdh = value.as_ref();
+        let grp = EcGroup::from_curve_name(Nid::SECP521R1)?;
+        let x = BigNum::from_slice(&ecdh[..ECDH_PUB_KEY_COORD_POINT_SIZE])?;
+        let y = BigNum::from_slice(&ecdh[ECDH_PUB_KEY_COORD_POINT_SIZE..])?;
+        let ec_key = EcKey::from_public_key_affine_coordinates(&grp, &x, &y)?;
+        Self::from_ec_key(ec_key)
+    }
+}
+
 macro_rules! ecdh_from {
     ($type: ty) => {
-        impl TryFrom<&PKeyRef<$type>> for EcdhPubkeyCoord {
+        impl TryFrom<&PKeyRef<$type>> for EcPubKeyCoord {
             type Error = ErrorStack;
 
             fn try_from(key: &PKeyRef<$type>) -> Result<Self, Self::Error> {
@@ -299,7 +337,16 @@ macro_rules! ecdh_from {
                 let grp = k.group();
                 let pub_key = k.public_key();
                 let coord = get_pub_ecdh_points(pub_key, grp)?;
-                Ok(EcdhPubkeyCoord(coord))
+                Ok(Self(coord))
+            }
+        }
+
+        impl TryFrom<PKey<$type>> for EcPubKeyCoord {
+            type Error = ErrorStack;
+
+            fn try_from(key: PKey<$type>) -> Result<Self, Self::Error> {
+                let key_ref = key.as_ref();
+                key_ref.try_into()
             }
         }
     };
@@ -385,7 +432,7 @@ pub trait Request {
 /// A struct to represent some parts of a binary/encrypted request.
 #[derive(Debug)]
 #[allow(clippy::len_without_is_empty)]
-pub struct BinReqValues<'a> {
+pub(crate) struct BinReqValues<'a> {
     iv: &'a [u8],
     aad: &'a [u8],
     req_dep_aad: &'a [u8],
@@ -395,13 +442,13 @@ pub struct BinReqValues<'a> {
     len: usize,
 }
 impl<'a> BinReqValues<'a> {
-    pub(crate) const TAG_LEN: usize = AES_256_GCM_TAG_SIZE;
+    pub(crate) const TAG_LEN: usize = SymKeyType::AES_256_GCM_TAG_LEN;
 
     /// Get the locations from this request.
     ///
     /// Does minimal sanity test, just tests to prevent panics.
     /// `req` may be larger than the actual request.
-    pub fn get(req: &'a [u8]) -> Result<Self> {
+    pub(crate) fn get(req: &'a [u8]) -> Result<Self> {
         let hdr = RequestHdr::read_from_prefix(req).ok_or(Error::BinRequestSmall)?;
         let rql = hdr.rql.get() as usize;
         let sea = hdr.sea.get() as usize;
@@ -432,23 +479,24 @@ impl<'a> BinReqValues<'a> {
     }
 
     /// Returns the version of this [`BinReqValues`].
-    pub fn version(&self) -> u32 {
+    pub(crate) fn version(&self) -> u32 {
         self.version
     }
 
     /// Returns the length of this [`BinReqValues`].
-    pub fn len(&self) -> usize {
+    pub(crate) fn len(&self) -> usize {
         self.len
     }
 
     /// Returns the size of the encrypted area
-    pub fn sea(&self) -> u32 {
+    pub(crate) fn sea(&self) -> u32 {
         self.encr.len() as u32
     }
 
     /// Decrypts the encrypted area with the provided key
-    pub fn decrypt(&self, key: &SymKey) -> Result<Confidential<Vec<u8>>> {
-        decrypt_aes_gcm(key, self.iv, self.aad, self.encr, self.tag)
+    pub(crate) fn decrypt(&self, key: &SymKey) -> Result<Confidential<Vec<u8>>> {
+        let result = decrypt_aead(key, self.iv, self.aad, self.encr, self.tag)?;
+        Ok(result.into_plain())
     }
 
     /// Returns a reference to the request dependent authenticated area of this [`BinReqValues`]
@@ -456,22 +504,23 @@ impl<'a> BinReqValues<'a> {
     ///
     /// If target struct is larger than the request depended-AAD None is returned. See
     /// [`FromBytes::ref_from_prefix`]
-    pub fn req_dep_aad<T>(&self) -> Option<&T>
+    pub(crate) fn req_dep_aad<T>(&self) -> Option<&T>
     where
         T: FromBytes + Sized,
     {
         T::ref_from_prefix(self.req_dep_aad)
     }
+
+    /// Returns a reference to the tag of this [`BinReqValues`].
+    pub(crate) fn tag(&self) -> &[u8] {
+        self.tag
+    }
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::get_test_asset;
-    use crate::request::SymKey;
-    use crate::test_utils::*;
-    use openssl::ec::EcGroup;
-    use openssl::nid::Nid;
+    use crate::{get_test_asset, request::SymKey, test_utils::*};
 
     static TEST_MAGIC: [u8; 8] = 0x12345689abcdef00u64.to_be_bytes();
 
@@ -595,19 +644,49 @@ mod tests {
     }
 
     #[test]
-    fn get_pub_ecdh_points() {
+    fn ec_pub_ec_coord_from() {
         let (cust_key, _) = get_test_keys();
-
         let pub_key = get_test_asset!("keys/public_cust.bin");
-
         assert_eq!(pub_key.len(), 160);
 
-        let points = cust_key.ec_key().unwrap();
-        let points = points.public_key();
-        let grp = EcGroup::from_curve_name(Nid::SECP521R1).unwrap();
+        let ec_coord: EcPubKeyCoord = cust_key.as_ref().try_into().unwrap();
+        assert_eq!(ec_coord.as_ref(), pub_key);
+    }
 
-        let points = super::get_pub_ecdh_points(points, &grp).unwrap();
+    #[test]
+    fn ec_pub_ec_coord_hash() {
+        let exp = [
+            0x5e, 0xe9, 0x05, 0xa9, 0xbe, 0x70, 0x36, 0x68, 0x15, 0xa4, 0x56, 0x41, 0xaf, 0xae,
+            0x00, 0x97, 0x3b, 0x1f, 0x45, 0x29, 0x2f, 0x43, 0xbc, 0xd7, 0x63, 0x8e, 0xe2, 0xa7,
+            0x3f, 0xd7, 0xc4, 0x5e,
+        ];
+        let (cust_key, _) = get_test_keys();
+        let ec_coord: EcPubKeyCoord = cust_key.as_ref().try_into().unwrap();
+        let hash = ec_coord.sha256().unwrap();
 
-        assert_eq!(&points, pub_key);
+        assert_eq!(hash.as_ref(), &exp);
+    }
+
+    #[test]
+    fn conversion_ecdh_and_vice_versa() {
+        let (_, cust_pub) = get_test_keys();
+        let phk: EcPubKeyCoord = cust_pub.clone().try_into().unwrap();
+
+        assert_eq!(
+            phk.as_ref(),
+            &[
+                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 118, 136, 28, 216, 75, 139, 109, 231,
+                18, 60, 126, 144, 14, 223, 120, 231, 247, 182, 132, 153, 145, 70, 177, 38, 59, 168,
+                184, 108, 132, 71, 240, 138, 182, 212, 105, 194, 177, 40, 237, 158, 28, 53, 1, 88,
+                5, 172, 211, 211, 2, 51, 211, 145, 34, 247, 226, 248, 170, 28, 43, 20, 123, 120,
+                131, 180, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 195, 69, 86, 194, 92,
+                249, 47, 41, 206, 102, 189, 68, 17, 77, 107, 123, 60, 120, 225, 58, 63, 144, 189,
+                185, 0, 64, 246, 135, 110, 82, 98, 247, 120, 166, 26, 147, 125, 27, 52, 128, 46,
+                178, 87, 227, 78, 6, 114, 221, 95, 42, 52, 122, 221, 170, 40, 32, 53, 9, 42, 112,
+                195, 92, 46, 121, 115
+            ]
+        );
+        let cust_pub_back: PKey<Public> = phk.try_into().unwrap();
+        assert!(cust_pub.public_eq(&cust_pub_back));
     }
 }
