@@ -1,3 +1,7 @@
+// SPDX-License-Identifier: MIT
+//
+// Copyright IBM Corp. 2024
+
 use crate::assert_size;
 use crate::{
     crypto::{sign_msg, verify_signature},
@@ -141,7 +145,7 @@ impl UserData {
         Ok(Self::Signed(SignedUserData { sign_key, data }))
     }
 
-    /// Signs data in buf, writes signature to buf+user_data_offset+sign_offset if applicable.
+    /// Signs data in buf, writes signature to `buf+user_data_offset+sign_offset` if applicable.
     ///
     /// Uses [`MessageDigest::sha512`] as digest. Does not modify the abritary user data buffer.
     ///
@@ -156,8 +160,8 @@ impl UserData {
     pub(super) fn sign(&self, buf: &mut [u8], user_data_offset: usize) -> Result<()> {
         // get signing info or return if no signature is required
         let signed_data = match self {
-            UserData::Null | UserData::Unsigned(_) => return Ok(()),
-            UserData::Signed(s) => s,
+            Self::Null | Self::Unsigned(_) => return Ok(()),
+            Self::Signed(s) => s,
         };
         debug_assert!(buf.len() >= USER_DATA_SIZE);
 
@@ -194,9 +198,9 @@ impl UserData {
     /// 512 bytes
     pub(super) fn data(&self) -> (Option<&[u8]>, Option<Vec<u8>>) {
         let buf = match self {
-            UserData::Null => None,
-            UserData::Unsigned(d) => Some(d),
-            UserData::Signed(SignedUserData { data, .. }) => Some(data),
+            Self::Null => None,
+            Self::Unsigned(d) => Some(d),
+            Self::Signed(SignedUserData { data, .. }) => Some(data),
         };
 
         let remaining_size = Self::USER_DATA_SIZE - buf.map(|b| b.len()).unwrap_or(0);
