@@ -1,17 +1,19 @@
 // SPDX-License-Identifier: MIT
 //
 // Copyright IBM Corp. 2023
-use crate::{
-    macros::{bail_spec, file_error},
-    Error, FileAccessErrorType, FileIoErrorType, Result,
-};
 use std::{
     fs::File,
     io::{Read, Write},
     path::Path,
 };
+
 use zerocopy::{AsBytes, BigEndian, FromBytes, FromZeroes, U64};
 
+use crate::{
+    macros::{bail_spec, file_error},
+    Error, FileAccessErrorType, FileIoErrorType, Result,
+};
+
 /// Trait that describes bitflags, represented by `T`.
 pub trait Flags<T>: From<T> + for<'a> From<&'a T> {
     /// Set the specified bit to one.
@@ -70,6 +72,12 @@ impl From<&u64> for Msb0Flags64 {
     }
 }
 
+impl From<Msb0Flags64> for u64 {
+    fn from(value: Msb0Flags64) -> Self {
+        value.0.into()
+    }
+}
+
 /// Bitflags in LSB0 ordering
 ///
 /// Wraps an u64 to set/get individual bits
@@ -112,6 +120,12 @@ impl From<&u64> for Lsb0Flags64 {
     }
 }
 
+impl From<Lsb0Flags64> for u64 {
+    fn from(value: Lsb0Flags64) -> Self {
+        value.0.into()
+    }
+}
+
 /// Tries to convert a BE hex string into a 128 unsigned integer
 /// The hexstring must contain 32chars of hexdigits
 ///
@@ -140,7 +154,7 @@ pub fn try_parse_u128(hex_str: &str, ctx: &str) -> Result<[u8; 16]> {
             "{ctx} hexstring must be 32chars long to cover all 16 bytes"
         ));
     }
-    parse_hex(hex_str).try_into().map_err(|_| {
+    decode_hex(hex_str)?.try_into().map_err(|_| {
         Error::Specification(format!(
             "{ctx} hexstring must be 32chars long to cover all 16 bytes"
         ))
@@ -184,6 +198,10 @@ pub fn try_parse_u64(hex_str: &str, ctx: &str) -> Result<u64> {
 /// Wraps [`File::open`]
 ///
 /// * `path` - Path to file
+///
+/// # Errors
+///
+/// This function will return errors according to [`File::open`].
 pub fn open_file<P: AsRef<Path>>(path: P) -> Result<File> {
     File::open(&path).map_err(|e| Error::FileAccess {
         ty: FileAccessErrorType::Open,
@@ -197,6 +215,10 @@ pub fn open_file<P: AsRef<Path>>(path: P) -> Result<File> {
 /// Wraps [`File::create`]
 ///
 /// * `path` - Path to file
+///
+/// # Errors
+///
+/// This function will return errors according to [`File::create`].
 pub fn create_file<P: AsRef<Path>>(path: P) -> Result<File> {
     File::create(&path).map_err(|e| Error::FileAccess {
         ty: FileAccessErrorType::Create,
@@ -214,8 +236,8 @@ pub fn create_file<P: AsRef<Path>>(path: P) -> Result<File> {
 /// If this function encounters an "end of file" before completely filling
 /// the buffer, it returns an error. The contents of `buf` are unspecified in this case.
 ///
-/// If aber so hast du mmmeny other read error is encountered then this function immediately
-/// returns. The contents of `buf` are unspecified in this case.
+/// If any other read error occurs, this function returns immediately. The
+/// contents of `buf` are unspecified in this case.
 ///
 /// If this function returns an error, it is unspecified how many bytes it
 /// has read, but it will never read more than would be necessary to
@@ -333,6 +355,28 @@ usize_to_ui! {
 #[doc = r"u16"]
 => u16, to_u16}
 
+/// Converts the hexstring into a byte vector.
+///
+/// # Errors
+///
+/// Raises an error if a non-hex character was found or the length was not a
+/// multiple of two.
+pub fn decode_hex<S: AsRef<str>>(s: S) -> Result<Vec<u8>> {
+    let hex = s.as_ref();
+    let hex_len = hex.len();
+    if hex_len % 2 != 0 {
+        return Err(Error::InvHexStringSize(hex_len));
+    }
+
+    (0..hex_len)
+        .step_by(2)
+        .map(|i| {
+            u8::from_str_radix(&hex[i..i + 2], 16)
+                .map_err(|err| Error::InvHexStringChar { source: err })
+        })
+        .collect()
+}
+
 /// Converts the hexstring into a byte vector.
 ///
 /// Stops if the end or until a non hex chat is found
@@ -475,6 +519,49 @@ mod tests {
         assert_eq!(super::parse_hex(s), exp);
     }
 
+    #[test]
+    fn decode_hex() {
+        let s = "123456acbef0";
+        let exp = vec![0x12, 0x34, 0x56, 0xac, 0xbe, 0xf0];
+        assert_eq!(super::decode_hex(s).expect("should not fail"), exp);
+
+        let s = "00123456acbef0";
+        let exp = vec![0, 0x12, 0x34, 0x56, 0xac, 0xbe, 0xf0];
+        assert_eq!(super::decode_hex(s).expect("should not fail"), exp);
+
+        let s = "00123456acbef0";
+        let exp = vec![0, 0x12, 0x34, 0x56, 0xac, 0xbe, 0xf0];
+        assert_eq!(super::decode_hex(s).expect("should not fail"), exp);
+
+        assert_eq!(
+            super::decode_hex("c0ffee").expect("should not fail"),
+            [0xc0, 0xff, 0xee]
+        );
+        assert_eq!(super::decode_hex("c0").expect("should not fail"), [0xc0]);
+        assert_eq!(super::decode_hex("").expect("should not fail"), []);
+
+        assert!(matches!(
+            super::decode_hex(" "),
+            Err(Error::InvHexStringSize(_))
+        ));
+        assert!(matches!(
+            super::decode_hex("coffee"),
+            Err(Error::InvHexStringChar { .. })
+        ));
+        assert!(matches!(
+            super::decode_hex(" c0a"),
+            Err(Error::InvHexStringChar { .. })
+        ));
+        assert!(matches!(
+            super::decode_hex("c0 a"),
+            Err(Error::InvHexStringChar { .. })
+        ));
+        assert!(matches!(
+            super::decode_hex("c0a"),
+            Err(Error::InvHexStringSize(_))
+        ));
+    }
+
     #[test]
     fn to_u32() {
         assert_eq!(Some(17), super::to_u32(17));
@@ -504,7 +591,7 @@ mod tests {
         ));
         assert!(matches!(
             try_parse_u128("-1223344556677889900aabbccddeeff", ""),
-            Err(Error::Specification(_))
+            Err(Error::InvHexStringChar { .. })
         ));
 
         assert!(matches!(
@@ -512,7 +599,7 @@ mod tests {
             Err(Error::Specification(_))
         ));
         assert!(matches!(
-            try_parse_u128("-0x1234", ""),
+            try_parse_u128("0x123", ""),
             Err(Error::Specification(_))
         ));
         assert!(matches!(
@@ -525,7 +612,7 @@ mod tests {
         ));
         assert!(matches!(
             try_parse_u128("0x-1223344556677889900aabbccddeeff", ""),
-            Err(Error::Specification(_))
+            Err(Error::InvHexStringChar { .. })
         ));
 
         assert_eq!(
