@@ -96,8 +96,8 @@ pub struct ComponentPaths {
 #[command(group(ArgGroup::new("header-flags").multiple(true).conflicts_with_all(["x_pcf", "x_scf"])))]
 pub struct CreateBootImageLegacyFlags {
     /// Enable Secure Execution guest dump support. This option requires the
-    /// '--comm-key' option.
-    #[arg(long, action = clap::ArgAction::SetTrue, requires="comm_key", group="header-flags")]
+    /// '--cck' option.
+    #[arg(long, action = clap::ArgAction::SetTrue, requires="cck", group="header-flags")]
     pub enable_dump: Option<bool>,
 
     /// Disable Secure Execution guest dump support (default).
@@ -105,9 +105,9 @@ pub struct CreateBootImageLegacyFlags {
     pub disable_dump: Option<bool>,
 
     /// Add-secret requests must provide an extension secret that matches the
-    /// CCK-derived extension secret. This option requires the '--comm-key'
+    /// CCK-derived extension secret. This option requires the '--cck'
     /// option.
-    #[arg(long, action = clap::ArgAction::SetTrue, requires="comm_key", group="header-flags")]
+    #[arg(long, action = clap::ArgAction::SetTrue, requires="cck", group="header-flags")]
     pub enable_cck_extension_secret: Option<bool>,
 
     /// Add-secret requests don't have to provide the CCK-derived extension
@@ -140,6 +140,20 @@ pub struct CreateBootImageLegacyFlags {
     /// Disable the support for backup target keys (default).
     #[arg(long, action = clap::ArgAction::SetTrue, conflicts_with="enable_backup_keys", group="header-flags")]
     pub disable_backup_keys: Option<bool>,
+
+    /// Enable encryption of the image components (default).
+    ///
+    /// The image components are: the kernel, ramdisk, and kernel command line.
+    #[arg(long, action = clap::ArgAction::SetTrue, group="header-flags")]
+    pub enable_image_encryption: Option<bool>,
+
+    /// Disable encryption of the image components.
+    ///
+    /// The image components are: the kernel, ramdisk, and kernel command line.
+    /// Use only if the components used do not contain any confidential content
+    /// (for example, secrets like non-public cryptographic keys).
+    #[arg(long, action = clap::ArgAction::SetTrue, conflicts_with="enable_image_encryption", group="header-flags")]
+    pub disable_image_encryption: Option<bool>,
 }
 
 #[non_exhaustive]
@@ -178,8 +192,11 @@ pub struct InfoArgs {
     pub format: OutputFormat,
 
     /// Use the key in FILE to decrypt the Secure Execution header.
-    #[arg(long, value_name = "FILE", value_hint = ValueHint::FilePath,)]
-    pub key: Option<PathBuf>,
+    ///
+    /// It is the key that was specified with the command line option
+    /// '--hdr-key' at the Secure Execution image creation.
+    #[arg(long, value_name = "FILE", value_hint = ValueHint::FilePath, alias = "key")]
+    pub hdr_key: Option<PathBuf>,
 }
 
 #[derive(Args, Debug)]
@@ -313,9 +330,18 @@ pub struct CreateBootImageArgs {
 
     /// Use the content of FILE as the customer-communication key (CCK).
     ///
-    /// The file must contain exactly 32 bytes of data.
-    #[arg(long, value_name = "FILE")]
-    pub comm_key: Option<PathBuf>,
+    /// The file must contain exactly 32 bytes of data. This option used to be
+    /// called '--comm-key' in previous versions.
+    #[arg(long, value_name = "FILE", visible_alias = "comm-key")]
+    pub cck: Option<PathBuf>,
+
+    /// Use the content of FILE as the Secure Execution header protection key.
+    ///
+    /// The file must contain exactly 32 bytes of data. If the option is not
+    /// specified, the Secure Execution header protection key is a randomly
+    /// generated key.
+    #[arg(long, value_name = "FILE", alias = "x-header-key")]
+    pub hdr_key: Option<PathBuf>,
 
     #[clap(flatten)]
     pub legacy_flags: CreateBootImageLegacyFlags,
@@ -338,11 +364,6 @@ pub struct CreateBootImageExperimentalArgs {
     #[arg(long, value_name = "FILE", hide(true))]
     pub x_comp_key: Option<PathBuf>,
 
-    /// Manually set the Secure Execution header protection key (experimental option).
-    // Hidden in user documentation.
-    #[arg(long, value_name = "FILE", hide(true))]
-    pub x_header_key: Option<PathBuf>,
-
     /// Manually set the PSW address used for the Secure Execution header (experimental option).
     // Hidden in user documentation.
     #[arg(long, value_name = "ADDRESS", hide(true))]
@@ -467,6 +488,8 @@ mod test {
             flat_map_collect(insert(mvca.clone(), vec![CliOption::new("parmfile", ["--parmfile", "/dev/null"])])),
             flat_map_collect(insert(mvca.clone(), vec![CliOption::new("enable-dump", ["--enable-dump"]),
                                                    CliOption::new("comm-key", ["--comm-key", "/dev/null"])])),
+            flat_map_collect(insert(mvca.clone(), vec![CliOption::new("enable-dump", ["--enable-dump"]),
+                                                   CliOption::new("comm-key", ["--cck", "/dev/null"])])),
             flat_map_collect(insert(mvca.clone(), vec![CliOption::new("enable-dump", ["--enable-dump"]),
                                                    CliOption::new("comm-key", ["--comm-key", "/dev/null"])])),
             flat_map_collect(insert(mvca.clone(), vec![CliOption::new("x-pcf", ["--x-pcf", "0x0"]),
@@ -476,6 +499,10 @@ mod test {
             flat_map_collect(insert(mvca.clone(), vec![CliOption::new("enable-pckmo", ["--enable-pckmo"])])),
             flat_map_collect(insert(mvca.clone(), vec![CliOption::new("enable-pckmo-hmac", ["--enable-pckmo-hmac"])])),
             flat_map_collect(insert(mvca.clone(), vec![CliOption::new("enable-backup-keys", ["--enable-backup-keys"])])),
+            flat_map_collect(insert(mvca.clone(), vec![CliOption::new("disable-image-encryption", ["--disable-image-encryption"])])),
+            flat_map_collect(insert(mvca.clone(), vec![CliOption::new("enable-image-encryption", ["--enable-image-encryption"])])),
+            flat_map_collect(insert(mvca.clone(), vec![CliOption::new("x-header-key", ["--x-header-key", "/dev/null"]),])),
+            flat_map_collect(insert(mvca.clone(), vec![CliOption::new("x-header-key", ["--hdr-key", "/dev/null"]),])),
         ];
         let invalid_create_args = [
             flat_map_collect(remove(mvcanv.clone(), "no-verify")),
@@ -501,6 +528,9 @@ mod test {
                                                    CliOption::new("x-pcf2", ["--x-pcf", "0x0"])])),
             flat_map_collect(insert(mvca.clone(), vec![CliOption::new("enable-pckmo", ["--enable-pckmo"]),
                                                    CliOption::new("disable-pckmo", ["--disable-pckmo"])])),
+            flat_map_collect(insert(mvca.clone(), vec![CliOption::new("enable-image-encryption", ["--enable-image-encryption"]),
+                                                   CliOption::new("disable-image-encryption", ["--disable-image-encryption"])])),
+            flat_map_collect(insert(mvca.clone(), vec![CliOption::new("x-header-key", ["--hdr-key"]),])),
         ];
 
         let mut genprotimg_valid_args = vec![
@@ -692,6 +722,22 @@ mod test {
                     CliOption::new("image", ["/dev/null"]),
                 ],
             )),
+            flat_map_collect(insert(
+                args.clone(),
+                vec![
+                    CliOption::new("hdr-key", ["--hdr-key", "/dev/null"]),
+                    CliOption::new("format", ["--format=json"]),
+                    CliOption::new("image", ["/dev/null"]),
+                ],
+            )),
+            flat_map_collect(insert(
+                args.clone(),
+                vec![
+                    CliOption::new("hdr-key", ["--key", "/dev/null"]),
+                    CliOption::new("format", ["--format=json"]),
+                    CliOption::new("image", ["/dev/null"]),
+                ],
+            )),
             // separation between keyword and positional args works
             flat_map_collect(insert(
                 args.clone(),
@@ -732,7 +778,7 @@ mod test {
 
         // Test for invalid combinations
         // Input is missing
-        let mut pvimg_invalid_args = vec![vec!["pvimg", "test"]];
+        let mut pvimg_invalid_args = vec![vec!["pvimg", "info"]];
 
         for create_args in &valid_test_args {
             pvimg_valid_args.push(
