@@ -1,7 +1,10 @@
 // SPDX-License-Identifier: MIT
 //
-// Copyright IBM Corp. 2023
+// Copyright IBM Corp. 2023, 2024
 
+use std::fmt::Display;
+
+use clap::error::ErrorKind::ValueValidation;
 use clap::{ArgGroup, Args, CommandFactory, Parser, Subcommand, ValueEnum, ValueHint};
 use utils::{CertificateOptions, DeprecatedVerbosityOptions, STDOUT};
 
@@ -37,8 +40,8 @@ pub struct CreateSecretOpt {
 
     /// Specifies the header of the guest image.
     ///
-    /// Can be an IBM Secure Execution image created by genprotimg or an extracted IBM Secure
-    /// Execution header. The header must start at a page boundary.
+    /// Can be an IBM Secure Execution image created by 'pvimg/genprotimg' or an
+    /// extracted IBM Secure Execution header.
     #[arg(long, value_name = "FILE", value_hint = ValueHint::FilePath)]
     pub hdr: String,
 
@@ -138,6 +141,12 @@ pub struct CreateSecretOpt {
     /// by default.
     #[arg(long, value_name = "FILE", value_hint = ValueHint::FilePath,)]
     pub user_sign_key: Option<String>,
+
+    /// Do not hash the name, use it directly as secret ID.
+    ///
+    /// Ignored for meta-secrets.
+    #[arg(long)]
+    pub use_name: bool,
 }
 
 #[derive(Subcommand, Debug)]
@@ -150,12 +159,12 @@ pub enum AddSecretType {
 
     /// Create an association secret.
     ///
-    /// Use an association secret to connect a trusted I/O device to a guest. The `pvapconfig` tool
+    /// Use an association secret to connect a trusted I/O device to a guest. The 'pvapconfig' tool
     /// provides more information about association secrets.
     Association {
-        /// String to identify the new secret.
+        /// String that identifies the new secret.
         ///
-        /// The actual secret is set with --input-secret. The name is saved in `NAME.yaml` with
+        /// The actual secret is set with '--input-secret'. The name is saved in `NAME.yaml` with
         /// white-spaces mapped to `_`.
         name: String,
 
@@ -166,17 +175,83 @@ pub enum AddSecretType {
         stdout: bool,
 
         /// Path from which to read the plaintext secret. Uses a random secret if not specified.
-        #[arg(long, value_name = "FILE", value_hint = ValueHint::FilePath, conflicts_with("output_secret"))]
+        #[arg(long, value_name = "SECRET-FILE", value_hint = ValueHint::FilePath, conflicts_with("output_secret"))]
         input_secret: Option<String>,
 
-        /// Save the generated secret as plaintext in FILE.
+        /// Save the generated secret as plaintext in SECRET-FILE.
         ///
         /// The generated secret can be used to generate add-secret requests for a different guest
-        /// with the same secret using --input-secret. Destroy the secret when it is not used
+        /// with the same secret using '--input-secret'. Destroy the secret when it is not used
         /// anymore.
-        #[arg(long, value_name = "FILE", value_hint = ValueHint::FilePath,)]
+        #[arg(long, value_name = "SECRET-FILE", value_hint = ValueHint::FilePath,)]
         output_secret: Option<String>,
     },
+
+    /// Create a retrievable secret.
+    ///
+    /// A retrievable secret is stored in the  per-guest storage of the Ultravisor. A SE-guest can
+    /// retrieve the secret at runtime and use it. All retrievable secrets, but the plaintext
+    /// secret, are retrieved as wrapped/protected key objects and only usable inside the current,
+    /// running SE-guest instance.
+    #[command(visible_alias = "retr")]
+    Retrievable {
+        /// String that identifies the new secret.
+        ///
+        /// The actual secret is set with '--secret'. The name is saved in `NAME.yaml` with
+        /// white-spaces mapped to `_`.
+        name: String,
+
+        /// Print the hashed name to stdout.
+        ///
+        /// The hashed name is not written to `NAME.yaml`
+        #[arg(long)]
+        stdout: bool,
+
+        /// Use SECRET-FILE as retrievable secret
+        #[arg(long, value_name = "SECRET-FILE", value_hint = ValueHint::FilePath)]
+        secret: String,
+
+        /// Specify the secret type.
+        ///
+        /// Limitations to the input data apply depending on the secret type.
+        #[arg(long = "type", value_name = "TYPE")]
+        kind: RetrieveableSecretInpKind,
+    },
+}
+
+#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, ValueEnum, Debug)]
+pub enum RetrieveableSecretInpKind {
+    /// A plaintext secret.
+    /// Can be any file up to 8190 bytes long
+    Plain,
+    /// An AES key.
+    /// Must be a plain byte file 128, 192, or 256 bit long.
+    Aes,
+    /// An AES-XTS key.
+    /// Must be a plain byte file 512, or 1024 bit long.
+    AesXts,
+    /// A HMAC-SHA key.
+    /// Must be a plain byte file 512, or 1024 bit long.
+    HmacSha,
+    /// An elliptic curve private key.
+    /// Must be a PEM or DER file.
+    Ec,
+}
+
+impl Display for RetrieveableSecretInpKind {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(
+            f,
+            "{}",
+            match self {
+                Self::Plain => "PLAINTEXT",
+                Self::Aes => "AES KEY",
+                Self::AesXts => "AES-XTS KEY",
+                Self::HmacSha => "HMAC-SHA KEY",
+                Self::Ec => "EC PRIVATE KEY",
+            }
+        )
+    }
 }
 
 // all members s390x only
@@ -238,18 +313,70 @@ pub struct VerifyOpt {
     pub output: String,
 }
 
+// all members s390x only
+#[derive(Args, Debug)]
+pub struct RetrSecretOptions {
+    /// Specify the secret ID to be retrieved.
+    ///
+    /// Input type depends on '--inform'. If `yaml` (default) is specified, it must be a yaml
+    /// created by the create subcommand of this tool. If `hex` is specified, it must be a hex
+    /// 32-byte unsigned big endian number string. Leading zeros are required.
+    #[cfg(target_arch = "s390x")]
+    #[arg(value_name = "ID", value_hint = ValueHint::FilePath)]
+    pub input: String,
+
+    /// Specify the output path to place the secret value
+    #[cfg(target_arch = "s390x")]
+    #[arg(short, long, value_name = "FILE", default_value = STDOUT, value_hint = ValueHint::FilePath)]
+    pub output: String,
+
+    /// Define input type for the Secret ID
+    #[cfg(target_arch = "s390x")]
+    #[arg(long, value_enum, default_value_t)]
+    pub inform: RetrInpFmt,
+
+    /// Define the output format for the retrieved secret
+    #[cfg(target_arch = "s390x")]
+    #[arg(long, value_enum, default_value_t)]
+    pub outform: RetrOutFmt,
+}
+
+#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, ValueEnum, Debug, Default)]
+pub enum RetrInpFmt {
+    /// Use a yaml file
+    #[default]
+    Yaml,
+    /// Use a hex string.
+    Hex,
+    /// Use a name-string. Will hash it if no secret with the name found.
+    Name,
+}
+
+#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, ValueEnum, Debug, Default)]
+pub enum RetrOutFmt {
+    /// Write the secret as PEM.
+    ///
+    /// File starts with `-----BEGIN IBM PROTECTED KEY----` and `-----BEGIN
+    /// PLAINTEXT SECRET-----` for plaintext secrets it contains one header
+    /// line with the type information and the base64 protected key
+    #[default]
+    Pem,
+    /// Write the secret in binary.
+    Bin,
+}
+
 #[derive(Subcommand, Debug)]
 pub enum Command {
     /// Create a new add-secret request.
     ///
     /// Create add-secret requests for IBM Secure Execution guests. Only create these requests in a
-    /// trusted environment, such as your workstation. The `pvattest create` command creates a
+    /// trusted environment, such as your workstation. The 'pvattest create' command creates a
     /// randomly generated key to protect the request. The generated requests can then be added on
-    /// an IBM Secure Execution guest using `pvsecret add`. The guest can then use the secrets with
+    /// an IBM Secure Execution guest using 'pvsecret add'. The guest can then use the secrets with
     /// the use case depending on the secret type.
     Create(Box<CreateSecretOpt>),
 
-    /// Perform an add-secret request (s390x only).
+    /// Submit an add-secret request to the Ultravisor (s390x only).
     ///
     /// Perform an add-secret request using a previously generated add-secret request. Only
     /// available on s390x.
@@ -258,7 +385,7 @@ pub enum Command {
     /// Lock the secret-store (s390x only).
     ///
     /// Lock the secret store (s390x only). After this command executed successfully, all
-    /// add-secret requests will fail. Only available on s390x.
+    /// subsequent add-secret requests will fail. Only available on s390x.
     Lock,
 
     /// List all ultravisor secrets (s390x only).
@@ -274,6 +401,10 @@ pub enum Command {
     /// provided key. Outputs the arbitrary user-data.
     Verify(VerifyOpt),
 
+    /// Retrieve a secret from the UV secret store (s390x only).
+    #[command(visible_alias = "retr")]
+    Retrieve(RetrSecretOptions),
+
     /// Print version information and exit.
     #[command(aliases(["--version"]), hide(true))]
     Version,
@@ -294,13 +425,13 @@ pub fn validate_cli(cli: &CliOptions) -> Result<(), clap::Error> {
             }
             if secret_out == &Some(format!("{name}.yaml")) {
                 return Err(CliOptions::command().error(
-                    clap::error::ErrorKind::ValueValidation,
+                    ValueValidation,
                     format!("Secret output file and the secret name '{name}.yaml' are the same."),
                 ));
             }
             if format!("{name}.yaml") == opt.output {
                 return Err(CliOptions::command().error(
-                    clap::error::ErrorKind::ValueValidation,
+                    ValueValidation,
                     format!(
                         "output file and the secret name '{}' are the same.",
                         &opt.output
