@@ -4,7 +4,6 @@
 
 use std::path::Path;
 
-use crate::cli::{AddSecretType, CreateSecretFlags, CreateSecretOpt};
 use anyhow::{anyhow, bail, Context, Error, Result};
 use log::{debug, info, trace, warn};
 use pv::{
@@ -22,6 +21,8 @@ use pv::{
 use serde_yaml::Value;
 use utils::get_writer_from_cli_file_arg;
 
+use crate::cli::{AddSecretType, CreateSecretFlags, CreateSecretOpt, RetrieveableSecretInpKind};
+
 fn write_out<P, D>(path: &P, data: D, ctx: &str) -> pv::Result<()>
 where
     P: AsRef<Path>,
@@ -32,6 +33,23 @@ where
     Ok(())
 }
 
+fn retrievable(name: &str, secret: &str, kind: &RetrieveableSecretInpKind) -> Result<GuestSecret> {
+    let secret_data = read_file(secret, &format!("retrievable {kind}"))?.into();
+
+    match kind {
+        RetrieveableSecretInpKind::Plain => GuestSecret::plaintext(name, secret_data),
+        RetrieveableSecretInpKind::Aes => GuestSecret::aes(name, secret_data),
+        RetrieveableSecretInpKind::AesXts => GuestSecret::aes_xts(name, secret_data),
+        RetrieveableSecretInpKind::HmacSha => GuestSecret::hmac_sha(name, secret_data),
+        RetrieveableSecretInpKind::Ec => GuestSecret::ec(
+            name,
+            read_private_key(secret_data.value())
+                .with_context(|| format!("Cannot read {secret} as {kind} from PEM or DER"))?,
+        ),
+    }
+    .map_err(Error::from)
+}
+
 /// Prepare an add-secret request
 pub fn create(opt: &CreateSecretOpt) -> Result<()> {
     if pv_guest_bit_set() {
@@ -62,7 +80,7 @@ pub fn create(opt: &CreateSecretOpt) -> Result<()> {
     write_out(&opt.output, ser_asrbc, "add-secret request")?;
     info!("Successfully wrote the request to '{}'", &opt.output);
 
-    write_secret(&opt.secret, &asrcb, &opt.output)
+    write_secret(&opt.secret, asrcb.guest_secret(), &opt.output)
 }
 
 /// Read+parse the first key from the buffer.
@@ -76,7 +94,7 @@ fn read_private_key(buf: &[u8]) -> Result<PKey<Private>> {
 fn build_asrcb(opt: &CreateSecretOpt) -> Result<AddSecretRequest> {
     debug!("Build add-secret request");
 
-    let secret = match &opt.secret {
+    let mut secret = match &opt.secret {
         AddSecretType::Meta => GuestSecret::Null,
         AddSecretType::Association {
             name,
@@ -88,9 +106,14 @@ fn build_asrcb(opt: &CreateSecretOpt) -> Result<AddSecretRequest> {
             input_secret: None,
             ..
         } => GuestSecret::association(name, None)?,
+        AddSecretType::Retrievable {
+            name, secret, kind, ..
+        } => retrievable(name, secret, kind)?,
     };
     trace!("AddSecret: {secret:x?}");
 
+    opt.use_name.then(|| secret.no_hash_name());
+
     let mut flags = match &opt.pcf {
         Some(v) => (&try_parse_u64(v, "pcf")?).into(),
         None => AddSecretFlags::default(),
@@ -136,7 +159,9 @@ fn build_asrcb(opt: &CreateSecretOpt) -> Result<AddSecretRequest> {
         .as_ref()
         .map(|p| read_file(p, "User-signing key"))
         .transpose()?
-        .map(|buf| read_private_key(&buf))
+        .map(|buf| {
+            read_private_key(&buf).context("Cannot read {secret} as private key from PEM or DER")
+        })
         .transpose()?;
 
     if user_data.is_some() || user_key.is_some() {
@@ -206,54 +231,62 @@ fn read_cuid(asrcb: &mut AddSecretRequest, opt: &CreateSecretOpt) -> Result<()>
     Ok(())
 }
 
+// Write non confidential data (=name+id) to a yaml stdout
+fn write_yaml<P: AsRef<Path>>(
+    name: &str,
+    guest_secret: &GuestSecret,
+    stdout: &bool,
+    outp_path: P,
+) -> Result<()> {
+    debug!("Non-confidential secret information: {guest_secret:x?}");
+
+    let secret_info = serde_yaml::to_string(guest_secret)?;
+    if stdout.to_owned() {
+        println!("{secret_info}");
+        return Ok(());
+    }
+
+    let gen_name: String = name
+        .chars()
+        .map(|c| if c.is_whitespace() { '_' } else { c })
+        .collect();
+    let mut yaml_path = outp_path
+        .as_ref()
+        .parent()
+        .with_context(|| format!("Cannot open directory of {:?}", outp_path.as_ref()))?
+        .to_owned();
+    yaml_path.push(gen_name);
+    yaml_path.set_extension("yaml");
+    write_out(&yaml_path, secret_info, "secret information")?;
+    warn!(
+        "Successfully wrote secret info to '{}'",
+        yaml_path.display().to_string()
+    );
+    Ok(())
+}
+
 /// Write the generated secret (if any) to the specified output stream
 fn write_secret<P: AsRef<Path>>(
     secret: &AddSecretType,
-    asrcb: &AddSecretRequest,
+    guest_secret: &GuestSecret,
     outp_path: P,
 ) -> Result<()> {
-    if let AddSecretType::Association {
-        name,
-        stdout,
-        output_secret: secret_out,
-        ..
-    } = secret
-    {
-        let gen_name: String = name
-            .chars()
-            .map(|c| if c.is_whitespace() { '_' } else { c })
-            .collect();
-        let mut gen_path = outp_path
-            .as_ref()
-            .parent()
-            .with_context(|| format!("Cannot open directory of {:?}", outp_path.as_ref()))?
-            .to_owned();
-        gen_path.push(format!("{gen_name}.yaml"));
-
-        // write non confidential data (=name+id) to a yaml
-        let secret_info = serde_yaml::to_string(asrcb.guest_secret())?;
-        if stdout.to_owned() {
-            println!("{secret_info}");
-        } else {
-            write_out(&gen_path, secret_info, "association secret info")?;
-            debug!(
-                "Non-confidential secret information: {:x?}",
-                asrcb.guest_secret()
-            );
-            warn!(
-                "Successfully wrote association info to '{}'",
-                gen_path.display()
-            );
-        }
-
-        if let Some(path) = secret_out {
-            if let GuestSecret::Association { secret, .. } = asrcb.guest_secret() {
-                write_out(path, secret.value(), "Association secret")?
-            } else {
-                unreachable!("The secret type has to be `association` at this point (bug)!")
+    match secret {
+        AddSecretType::Association {
+            name,
+            stdout,
+            output_secret,
+            ..
+        } => {
+            write_yaml(name, guest_secret, stdout, outp_path)?;
+            if let Some(path) = output_secret {
+                write_out(path, guest_secret.confidential(), "Association secret")?
             }
-            info!("Successfully wrote generated association secret to '{path}'");
         }
+        AddSecretType::Retrievable { name, stdout, .. } => {
+            write_yaml(name, guest_secret, stdout, outp_path)?
+        }
+        _ => (),
     };
     Ok(())
 }
