@@ -2,7 +2,7 @@
 #
 # dbginfo.sh - Tool to collect runtime, configuration, and trace information
 #
-# Copyright IBM Corp. 2002, 2024
+# Copyright IBM Corp. 2002, 2025
 #
 # s390-tools is free software; you can redistribute it and/or modify
 # it under the terms of the MIT license. See LICENSE for details.
@@ -38,6 +38,8 @@ readonly LOG_FILE_SIZE_CHECK=50  # max logfile size in MB
 readonly LOG_FILE_AGE_CHECK=7  # age in days to include for size checking
 # Mount point of the debug file system
 readonly MOUNT_POINT_DEBUGFS="/sys/kernel/debug"
+# Network devices
+readonly NETWORK_DEVS=$(cd /sys/class/net; ls -d */ 2>/dev/null |  sed 's/\///g')
 # distro info
 readonly OSPRETTY="$(cat /etc/os* 2>/dev/null | grep -m1 PRETTY_NAME | sed 's/\"//g')"
 readonly OS_NAME="${OSPRETTY##*=}"
@@ -81,7 +83,7 @@ paramWORKDIR_BASE="/tmp"  # initial default path
 print_version() {
 	cat <<EOF
 ${SCRIPTNAME}: Debug information script version %S390_TOOLS_VERSION%
-Copyright IBM Corp. 2002, 2024
+Copyright IBM Corp. 2002, 2025
 EOF
 }
 
@@ -117,8 +119,6 @@ Options:
         -v|--version       print version information
         -c|--check         online base system information (no data collection)
 
-Please report bugs to: linux390@de.ibm.com
-
 EOF
 }
 
@@ -374,6 +374,7 @@ LOGFILES="\
   /var/log/dracut.log*\
   /var/log/IBMtape.trace\
   /var/log/IBMtape.errorlog\
+  /var/log/kdump.log\
   /var/log/libvirt\
   /var/log/lin_tape.trace\
   /var/log/lin_tape.errorlog\
@@ -475,6 +476,7 @@ CMDS="uname -a\
   :last\
   :lscpu -ae\
   :lscpu -ye\
+  :lscpumf -i\
   :lsmem\
   :lsmod\
   :lsshut\
@@ -542,6 +544,8 @@ CMDS="${CMDS}\
 
 # crypto specific commands
 CMDS="${CMDS}\
+  :cpacfinfo # for MSA 13 info\
+  :cpacfinfo -m -f -a -n\
   :ep11info -D\
   :ep11info -H\
   :ep11info -M\
@@ -1014,20 +1018,20 @@ collect_initrd_configfiles() {
 
 ########################################
 collect_osaoat() {
-	local network_devices
-	local network_device
+	local osa_devices
+	local osa_device
 
-	network_devices=$(lsqeth 2>/dev/null | grep "Device name" \
-		     | sed 's/D.*:[[:space:]]*\([^[:space:]]*\)[[:space:]]\+/\1/g' \
-		     | sed 's/[()]//g' )
 	if type qethqoat >/dev/null; then
-		if test -n "${network_devices}"; then
+		osa_devices=$(lsqeth 2>/dev/null | grep "Device name" \
+			     | sed 's/D.*:[[:space:]]*\([^[:space:]]*\)[[:space:]]\+/\1/g' \
+			     | sed 's/[()]//g' )
+		if test -n "${osa_devices}"; then
 			pr_collect_output "osa oat"
-			for network_device in ${network_devices}; do
-				call_run_command "qethqoat ${network_device}" \
+			for osa_device in ${osa_devices}; do
+				call_run_command "qethqoat ${osa_device}" \
 					"${OUTPUT_FILE_OSAOAT}.out" &&
-				call_run_command "qethqoat -r ${network_device}" \
-					"${OUTPUT_FILE_OSAOAT}_${network_device}.raw"
+				call_run_command "qethqoat -r ${osa_device}" \
+					"${OUTPUT_FILE_OSAOAT}_${osa_device}.raw"
 			done
 		else
 			pr_skip "osa oat: no devices"
@@ -1039,14 +1043,12 @@ collect_osaoat() {
 
 ########################################
 collect_ethtool() {
-	local network_devices
 	local network_device
 
-	network_devices=$(ls /sys/class/net 2>/dev/null)
 	if type ethtool >/dev/null; then
-		if test -n "${network_devices}"; then
+		if test -n "${NETWORK_DEVS}"; then
 			pr_collect_output "ethtool"
-			for network_device in ${network_devices}; do
+			for network_device in ${NETWORK_DEVS}; do
 				call_run_command "ethtool ${network_device}" \
 					"${OUTPUT_FILE_ETHTOOL}"
 				call_run_command "ethtool -k ${network_device}" \
@@ -1082,14 +1084,12 @@ collect_ethtool() {
 
 ########################################
 collect_tc() {
-	local network_devices
 	local network_device
 
-	network_devices=$(ls /sys/class/net 2>/dev/null)
 	if type tc >/dev/null; then
-		if test -n "${network_devices}"; then
+		if test -n "${NETWORK_DEVS}"; then
 			pr_collect_output "Trafic Control"
-			for network_device in ${network_devices}; do
+			for network_device in ${NETWORK_DEVS}; do
 				call_run_command "tc -s qdisc show dev ${network_device}" \
 					"${OUTPUT_FILE_TC}"
 			done
@@ -1103,14 +1103,12 @@ collect_tc() {
 
 ########################################
 collect_bridge() {
-	local network_devices
 	local network_device
 
-	network_devices=$(ls /sys/class/net 2>/dev/null)
 	if type bridge >/dev/null; then
-		if test -n "${network_devices}"; then
+		if test -n "${NETWORK_DEVS}"; then
 			pr_collect_output "bridge"
-			for network_device in ${network_devices}; do
+			for network_device in ${NETWORK_DEVS}; do
 				call_run_command "bridge -d link show dev ${network_device}" \
 					"${OUTPUT_FILE_BRIDGE}"
 				call_run_command "bridge -s fdb show dev ${network_device}" \
