@@ -3,7 +3,7 @@
  *
  * Common functions for stand-alone dump tools
  *
- * Copyright IBM Corp. 2013, 2018
+ * Copyright IBM Corp. 2013, 2023
  *
  * s390-tools is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -13,12 +13,14 @@
 
 #include "lib/zt_common.h"
 #include "boot/error.h"
+#include "dump/s390_dump.h"
 
 #include "libc.h"
 #include "sclp.h"
 #include "stage2dump.h"
 #include "boot/ipl.h"
 #include "boot/os_info.h"
+#include "boot/s390.h"
 
 #define CPU_ADDRESS_MAX	1000
 #define MACHINE_HAS_VX	machine_has_vx
@@ -72,7 +74,6 @@ void progress_print(unsigned long addr)
 	}
 	if (time < next && addr != dump_hdr->mem_size)
 		return;
-
 	printf("%08lu / %08lu MB (Dump file size %08lu MB)", addr >> 20,
 	       dump_hdr->mem_size >> 20, total_dump_size >> 20);
 	next = time + delta;
@@ -215,13 +216,23 @@ static void df_s390_dump_init(void)
 	dh->hdr_size = DF_S390_HDR_SIZE;
 	dh->page_size = PAGE_SIZE;
 	dh->dump_level = 4;
-	dh->version = 1;
+	dh->version = 2;
 	dh->mem_start = 0;
 	dh->arch = DF_S390_ARCH_64;
 	dh->build_arch = DF_S390_ARCH_64;
 	get_cpu_id((struct cpuid *) &dh->cpu_id);
 	dh->tod = get_tod_clock();
 	dh->volnr = 0;
+	dh->zlib_version_s390 = 0;
+	/*
+	 * Ignore compression if --no-compress attribute has been specified
+	 * or DFLTCC facility not available.
+	 */
+	if (test_facility(DFLTCC_FACILITY) &&
+	    parm_tail.no_compress == 0) {
+		dh->zlib_version_s390 = 1; /* Indicate interlnal Zlib version */
+		dh->zlib_entry_size = DUMP_SEGM_ZLIB_ENTSIZE;
+	}
 }
 
 /*
@@ -444,10 +455,15 @@ void __noreturn start(void)
 	init_early();
 	dt_device_parm_setup();
 	sclp_setup(SCLP_INIT);
+	/* Store facility-list for future checks */
+	stfle(S390_lowcore.stfle_fac_list,
+	      ARRAY_SIZE(S390_lowcore.stfle_fac_list));
 	dt_device_enable();
 	df_s390_dump_init();
 	printf("zIPL v%s dump tool (64 bit)", RELEASE_STRING);
 	printf("Dumping 64 bit OS");
+	if (parm_tail.no_compress)
+		printf("Dump compression prevented by the --no-compress attribute");
 	mem_and_cpu_init();
 	store_status();
 	dt_dump_mem();
