@@ -21,6 +21,7 @@
 
 #define BOOTMAP_HEADER_VERSION  1
 
+#define PROGRAM_TABLE_BLOCK_SIZE  512
 #define SIGNATURE_MAGIC  "~Module signature appended~\n"
 #define	PKCS7_FORMAT 0x01
 
@@ -31,18 +32,6 @@ struct bootmap_header {
 	char reserved[448];
 };
 
-struct signature_header {
-	uint8_t format;
-	uint8_t reserved[3];
-	uint32_t length;
-} __packed;
-
-typedef union {
-	uint64_t load_address;
-	uint64_t load_psw;
-	struct signature_header sig_head;
-} component_data;
-
 /*
  * The file_signature structure and the PKEY_ID definition
  * are based on linux/scripts/sign-file.c
@@ -63,11 +52,7 @@ struct file_signature {
 int bootmap_header_init(int fd);
 int bootmap_header_read(int fd, struct bootmap_header *bh);
 int bootmap_header_write(int fd, struct bootmap_header *bh);
-int bootmap_create(struct job_data* job, disk_blockptr_t* program_table,
-		   disk_blockptr_t *scsi_dump_sb_blockptr,
-		   disk_blockptr_t** stage2_list, blocknum_t* stage2_count,
-		   char** device, struct disk_info** new_info);
-void bootmap_store_blockptr(void* buffer, disk_blockptr_t* ptr,
-			    struct disk_info* info);
+void bootmap_store_blockptr(void *buffer, disk_blockptr_t *ptr,
+			    struct disk_info *info, int fid);
 
 #endif /* if not BOOTMAP_H */
