@@ -22,8 +22,10 @@
 #include <sys/vfs.h>
 #include <unistd.h>
 #include <linux/fs.h>
+#include <linux/major.h>
 #include <linux/fiemap.h>
 #include <linux/nvme_ioctl.h>
+#include <linux/raid/md_u.h>
 #include <assert.h>
 
 #include "lib/util_proc.h"
@@ -35,12 +37,6 @@
 #include "job.h"
 #include "misc.h"
 
-/* from linux/fs.h */
-#define FIBMAP			_IO(0x00,1)
-#define FIGETBSZ		_IO(0x00,2)
-#define BLKGETSIZE		_IO(0x12,96)
-#define BLKSSZGET		_IO(0x12,104)
-
 /* from linux/hdregs.h */
 #define HDIO_GETGEO		0x0301
 
@@ -410,7 +406,7 @@ static int disk_set_info_auto(struct disk_info *data,
 			data->partnum = stats->st_rdev & DASD_PARTN_MASK;
 			data->device = stats->st_rdev & ~DASD_PARTN_MASK;
 		}
-	} else if (strcmp(data->drv_name, "dasd") == 0) {
+	} else if (strcmp(data->drv_name, UTIL_PROC_DEV_ENTRY_DASD) == 0) {
 		/* Driver name is 'dasd' */
 		if (ioctl(fd, BIODASDINFO, &dasd_info)) {
 			error_reason("Could not determine DASD type");
@@ -421,15 +417,15 @@ static int disk_set_info_auto(struct disk_info *data,
 			return -1;
 		data->partnum = stats->st_rdev & DASD_PARTN_MASK;
 		data->device = stats->st_rdev & ~DASD_PARTN_MASK;
-	} else if (strcmp(data->drv_name, "sd") == 0) {
+	} else if (strcmp(data->drv_name, UTIL_PROC_DEV_ENTRY_SD) == 0) {
 		/* Driver name is 'sd' */
 		data->devno = -1;
 		data->type = disk_type_scsi;
 		data->partnum = stats->st_rdev & SCSI_PARTN_MASK;
 		data->device = stats->st_rdev & ~SCSI_PARTN_MASK;
 
-	} else if (strcmp(data->drv_name, "virtblk") == 0) {
-
+	} else if (strcmp(data->drv_name, UTIL_PROC_DEV_ENTRY_VIRTBLK) == 0) {
+		/* Driver name is 'virtblk' */
 		if (ioctl(fd, HDIO_GETGEO, &data->geo) != 0)
 			perror("Could not retrieve disk geometry information.");
 		if (ioctl(fd, BLKSSZGET, &data->phy_block_size) != 0)
@@ -440,9 +436,9 @@ static int disk_set_info_auto(struct disk_info *data,
 				     "determined.");
 			return -1;
 		}
-	/* NVMe path, driver name is 'blkext' */
-	} else if (strcmp(data->drv_name, "blkext") == 0 &&
+	} else if (strcmp(data->drv_name, UTIL_PROC_DEV_ENTRY_BLKEXT) == 0 &&
 		   ioctl(fd, NVME_IOCTL_ID) >= 0) {
+		/* NVMe path, driver name is 'blkext' */
 		data->devno = -1;
 		data->type = disk_type_scsi;
 		data->is_nvme = 1;
@@ -492,7 +488,7 @@ static void set_source_type(struct job_target_data *td,
 	td->source = source_auto;
 }
 
-static void set_driver_name(struct disk_info *info, dev_t device)
+static void set_driver_name(int fd, struct disk_info *info, dev_t device)
 {
 	struct util_proc_dev_entry dev_entry;
 
@@ -500,7 +496,17 @@ static void set_driver_name(struct disk_info *info, dev_t device)
 		/* already set */
 		return;
 	if (util_proc_dev_get_entry(device, 1, &dev_entry) == 0) {
-		info->drv_name = misc_strdup(dev_entry.name);
+		mdu_array_info_t array;
+
+		if (strcmp(dev_entry.name, UTIL_PROC_DEV_ENTRY_BLKEXT) == 0 &&
+		    ioctl(fd, GET_ARRAY_INFO, &array) >= 0)
+			/*
+			 * Driver name is 'blkext',
+			 * it is actually an md-partition
+			 */
+			info->drv_name = misc_strdup(UTIL_PROC_DEV_ENTRY_MD);
+		else
+			info->drv_name = misc_strdup(dev_entry.name);
 		util_proc_dev_free_entry(&dev_entry);
 	} else {
 		fprintf(stderr, "Warning: Could not determine driver name for "
@@ -684,7 +690,7 @@ int disk_get_info(const char *device, struct job_target_data *td,
 	if (!data)
 		goto error;
 	memset((void *)data, 0, sizeof(struct disk_info));
-	set_driver_name(data, stats.st_rdev);
+	set_driver_name(fd, data, stats.st_rdev);
 	set_source_type(td, data->drv_name, &script_file);
 	switch (td->source) {
 	case source_script:
